<?php
/**
 * Copyright © 2015 FiloBlu . All rights reserved.
 */

namespace FiloBlu\Flow\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Swatches\Model\Swatch;

class Attribute extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * @var
     */
    protected $_connection;
    /**
     * @var
     */
    protected $_setup;

    /**
     * @var \Magento\Catalog\Api\ProductAttributeRepositoryInterface
     */
    protected $attributeRepository;

    /**
     * @var array
     */
    protected $attributeValues;

    /**
     * @var \Magento\Eav\Model\Entity\Attribute\Source\TableFactory
     */
    protected $tableFactory;

    /**
     * @var \Magento\Eav\Api\AttributeOptionManagementInterface
     */
    protected $attributeOptionManagement;

    /**
     * @var \Magento\Eav\Api\Data\AttributeOptionLabelInterfaceFactory
     */
    protected $optionLabelFactory;

    /**
     * @var \Magento\Eav\Api\Data\AttributeOptionInterfaceFactory
     */
    protected $optionFactory;

    /**
     * @var Monolog\Logger
     */
    protected $_logger;

    /**
     * @var \Magento\Swatches\Helper\Data
     */
    protected $swatchHelper;

    public function __construct(
        \Magento\Framework\App\Helper\Context $contenxt,
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Catalog\Api\ProductAttributeRepositoryInterface $attributeRepository,
        \Magento\Eav\Model\Entity\Attribute\Source\TableFactory $tableFactory,
        \Magento\Eav\Api\AttributeOptionManagementInterface $attributeOptionManagement,
        \Magento\Eav\Api\Data\AttributeOptionLabelInterfaceFactory $optionLabelFactory,
        \Magento\Eav\Api\Data\AttributeOptionInterfaceFactory $optionFactory,
        \Magento\Swatches\Helper\Data $swatchHelper
    )
    {
        $this->_logger = $loggerProvider->getLogger();
        parent::__construct($contenxt);

        $this->attributeRepository = $attributeRepository;
        $this->tableFactory = $tableFactory;
        $this->attributeOptionManagement = $attributeOptionManagement;
        $this->optionLabelFactory = $optionLabelFactory;
        $this->optionFactory = $optionFactory;
        $this->swatchHelper = $swatchHelper;
    }

    public function addAttributeTranslation($option)
    {
        $optionId = $option['option_id'];
        $optionValue = $option['value'];
        $storeId = $option['store_id'];

        if (empty($optionValue)) {
            throw new \Exception("Option value cannot be empty");
        }

        if (empty($storeId)) {
            throw new \Exception("I can't add translation for default store view");
        }

        if (empty($optionId)) {
            throw new \Exception("I can't add translation for unknown option");
        }

        $optionValueTable = 'eav_attribute_option_value';
        //we'd might update
        $select = $this->getConnection()->select()
            ->from($optionValueTable)
            ->where('option_id = ' . $optionId)
            ->where('store_id = ' . $storeId);

        $result = $this->getConnection()->fetchRow($select);
        if ($result) {
            $where = array('option_id =?' => $option['option_id'],
                'store_id =?' => $option['store_id']);
            $this->getConnection()->update($optionValueTable, $option, $where);
            return true;
        }
        $this->getConnection()->insert($optionValueTable, $option);
        return true;
    }

    /*
     * Swatch type: 0 - text, 1 - visual color, 2 - visual image
     */

    protected function getConnection()
    {
        if (empty($this->_connection)) {
            $this->_connection = $this->getSetup()->getConnection(\Magento\Framework\App\ResourceConnection::DEFAULT_CONNECTION);
        }
        return $this->_connection;
    }

    protected function getSetup()
    {
        if (empty($this->_setup)) {
            $this->_setup = \Magento\Framework\App\ObjectManager::getInstance()->get('\Magento\Framework\App\ResourceConnection');
        }
        return $this->_setup;
    }

    /**
     * Find or create a matching attribute option
     *
     * @param string $attributeCode Attribute the option should exist in
     * @param string $label Label to find or add
     * @return int
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function createOrGetId($attributeCode, $label)
    {
        $label = trim($label);
        if (strlen($label) < 1) {
            throw new \Magento\Framework\Exception\LocalizedException(
                __('Label for %1 must not be empty.', $attributeCode)
            );
        }

        // Does it already exist?
        $optionId = $this->getOptionId($attributeCode, $label);

        if (!$optionId) {
            // If no, add it.

            /** @var \Magento\Eav\Model\Entity\Attribute\OptionLabel $optionLabel */
            $optionLabel = $this->optionLabelFactory->create();
            $optionLabel->setStoreId(0);
            $optionLabel->setLabel($label);

            $option = $this->optionFactory->create();
            $option->setLabel($optionLabel);
            $option->setStoreLabels([$optionLabel]);
            $option->setSortOrder(0);
            $option->setIsDefault(false);

            $this->attributeOptionManagement->add(
                \Magento\Catalog\Model\Product::ENTITY,
                $this->getAttribute($attributeCode)->getAttributeId(),
                $option
            );

            // Get the inserted ID. Should be returned from the installer, but it isn't.
            $optionId = $this->getOptionId($attributeCode, $label, true);
        }

        // Still can't insert option? Try direct sql then
        if (!$optionId) {
            $option_direct = array();
            $option_direct['attribute_id'] = $this->getAttribute($attributeCode)->getAttributeId();
            $option_direct['value'] = $label;
            $option_direct['store_id'] = 0;
            $this->addAttributeOption($option_direct);
            // Get the inserted ID. Should be returned from the installer, but it isn't.
            $optionId = $this->getOptionId($attributeCode, $label, true);
        }

        if ($optionId && $this->swatchHelper->isSwatchAttribute($this->getAttribute($attributeCode))) {
            $this->addSwatchOption($optionId, $this->getAttribute($attributeCode), $label);
        }

        return $optionId;
    }

    /**
     * Find the ID of an option matching $label, if any.
     *
     * @param string $attributeCode Attribute code
     * @param string $label Label to find
     * @param bool $force If true, will fetch the options even if they're already cached.
     * @return int|false
     */
    public function getOptionId($attributeCode, $label, $force = false)
    {
        /** @var \Magento\Catalog\Model\ResourceModel\Eav\Attribute $attribute */
        $attribute = $this->getAttribute($attributeCode);
        $attribute->setStoreId(0);

        // Build option array if necessary
        if ($force === true || !isset($this->attributeValues[$attribute->getAttributeId()])) {
            $this->attributeValues[$attribute->getAttributeId()] = [];

            // We have to generate a new sourceModel instance each time through to prevent it from
            // referencing its _options cache. No other way to get it to pick up newly-added values.

            /** @var \Magento\Eav\Model\Entity\Attribute\Source\Table $sourceModel */
            $sourceModel = $this->tableFactory->create();
            $sourceModel->setAttribute($attribute);

            foreach ($sourceModel->getAllOptions() as $option) {
                $this->attributeValues[$attribute->getAttributeId()][$option['label']] = $option['value'];
            }
        }

        // Return option ID if exists
        if (isset($this->attributeValues[$attribute->getAttributeId()][$label])) {
            return $this->attributeValues[$attribute->getAttributeId()][$label];
        }

        // Return false if does not exist
        return false;
    }

    /**
     * Get attribute by code.
     *
     * @param string $attributeCode
     * @return \Magento\Catalog\Api\Data\ProductAttributeInterface
     */
    public function getAttribute($attributeCode)
    {
        return $this->attributeRepository->get($attributeCode);
    }

    public function addAttributeOption($option)
    {

        $optionTable = 'eav_attribute_option';
        $attributeId = $option['attribute_id'];
        $optionValue = $option['value'];
        $storeId = $option['store_id'];

        if (empty($optionValue)) {
            throw new \Exception("Option value cannot be empty");
        }

        if ($storeId != 0) {
            throw new \Exception("I can only add option for admin store view");
        }

        if (empty($attributeId)) {
            throw new \Exception("Attribute id has to be defined when adding an option");
        }
        // add option
        $data = array(
            'attribute_id' => $option['attribute_id']
        );
        $this->getConnection()->insert($optionTable, $data);
        $optionId = $this->getConnection()->lastInsertId($optionTable);
        if (empty($optionId)) {
            return false;
        }
        $optionValueTable = 'eav_attribute_option_value';
        $option['option_id'] = $optionId;
        unset($option['attribute_id']);
        $this->getConnection()->insert($optionValueTable, $option);
        return $optionId;
    }

    /**
     * @param $optionId
     * @param $attribute
     * @param $value
     * @param int $store
     * @return bool
     */
    public function addSwatchOption($optionId, $attribute, $value, $store = 0)
    {

        if (!$this->swatchHelper->isSwatchAttribute($attribute)) {
            return false;
        }

        $type = Swatch::SWATCH_TYPE_EMPTY;

        switch (true) {
            case $this->swatchHelper->isTextSwatch($attribute):
                $type = Swatch::SWATCH_TYPE_TEXTUAL;
                break;

            case $this->swatchHelper->isVisualSwatch($attribute):

                if ($attribute->getData('use_product_image_for_swatch') !== 0) {
                    $type = Swatch::SWATCH_TYPE_VISUAL_IMAGE;
                } else {
                    $type = Swatch::SWATCH_TYPE_VISUAL_COLOR;
                }
                break;

            default:
                $type = Swatch::SWATCH_TYPE_EMPTY;
        }

        $attributeTable = $this->getConnection()->getTableName('eav_attribute_option_swatch');

        //Fix query failing for values of type string
        $sql = "INSERT INTO $attributeTable (`option_id`, `store_id`, `type`, `value`) VALUES(?, ?, ?, ?) ON DUPLICATE KEY UPDATE type=?, value=?";

        //Execute query prepared
        $this->getConnection()->query($sql, [$optionId, $store, $type, $value, $type, $value]);
    }

}
